Package: phenopath
Type: Package
Title: Genomic trajectories with heterogeneous genetic and
        environmental backgrounds
Version: 1.34.0
Date: 2017-10-24
Author: Kieran Campbell
Maintainer: Kieran Campbell <kieranrcampbell@gmail.com>
Description: PhenoPath infers genomic trajectories (pseudotimes) in the presence of
  heterogeneous genetic and environmental backgrounds and tests for interactions 
  between them.
License: Apache License (== 2.0)
Imports: Rcpp (>= 0.12.8), SummarizedExperiment, methods, stats, dplyr,
        tibble, ggplot2, tidyr
Suggests: knitr, rmarkdown, forcats, testthat, BiocStyle,
        SingleCellExperiment
biocViews: ImmunoOncology, RNASeq, GeneExpression, Bayesian,
        SingleCell, PrincipalComponent
LinkingTo: Rcpp
RoxygenNote: 6.0.1
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/phenopath
git_branch: RELEASE_3_22
git_last_commit: f09cdb5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 05:42:12 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

S3method(print,phenopath_fit)
export(clvm)
export(interaction_effects)
export(interaction_sds)
export(interactions)
export(phenopath)
export(plot_elbo)
export(significant_interactions)
export(simulate_phenopath)
export(trajectory)
import(Rcpp)
import(ggplot2)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(dplyr,mutate)
importFrom(methods,is)
importFrom(stats,coef)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,prcomp)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(tibble,as_data_frame)
importFrom(tidyr,gather)
useDynLib(phenopath)

## PhenoPath

PhenoPath learns genomic trajectories (pseudotimes) in the presence of heterogenous environmental and genetic backgrounds encoded as additional covariates and identifies interactions between the trajectories and covariates. Scalable variational Bayesian inference allows the trajectory and interactions to be inferred for thousands of samples and genes quickly.

[![Build Status](https://travis-ci.org/kieranrcampbell/phenopath.svg?branch=master)](https://travis-ci.org/kieranrcampbell/phenopath)

To quickly get started, see either of the vignettes:

* [Introduction to PhenoPath](https://kieranrcampbell.github.io/phenopath/introduction_to_phenopath.html)
* [End-to-end example on Shalek et al. (2014) from raw data to stimulant-pseudotime interactions](https://kieranrcampbell.github.io/phenopath/phenopath_shalek_vignette.html)

### Installation

```r
install.packages("devtools") # If not already installed
devtools::install_github("kieranrcampbell/phenopath", build_vignettes = TRUE)
```

### Overview

PhenoPath models the observed expression *y* in terms of a latent pathway score (pseudotime) *z*. Uniquely, the evolution of genes along the trajectory isn't common to each gene but can be perturbed by an additional sample-specific covariate. For example, this could be the mutational status of each sample or a drug that each sample was exposed to.

<img src="https://user-images.githubusercontent.com/2039489/26843141-1deacabc-4ae7-11e7-8630-a6a6e425ca0b.png" width="500"/>

#### Inference

Inference is performed using co-ordinate ascent variational inference (CAVI) implemented using the Rcpp library for increased performance. This minimises the KL-divergence between a set of approximating distributions and the true posterior by making a full-factorized mean-field approximation.


### Getting started

See the vignette for details.



### Authors

Kieran R Campbell & Christopher Yau

## ----load-packages, include=FALSE---------------------------------------------
knitr::opts_chunk$set(fig.width = 5, fig.height = 3)

suppressPackageStartupMessages({
  library(dplyr)
  library(dplyr)
  library(ggplot2)
  library(tidyr)
  library(forcats)
  library(phenopath)
})

## ----simulate-data------------------------------------------------------------
set.seed(123L)
sim <- simulate_phenopath()

## ----sim-structure------------------------------------------------------------
print(str(sim))

## ----some-genes, fig.width = 7------------------------------------------------
genes_to_extract <- c(1,3,11,13,21,23,31,33)
expression_df <- as.data.frame(sim$y[,genes_to_extract])
names(expression_df) <- paste0("gene_", genes_to_extract)

df_gex <- as_tibble(expression_df) %>% 
  mutate(x = factor(sim[['x']]), z = sim[['z']]) %>% 
  gather(gene, expression, -x, -z)

ggplot(df_gex, aes(x = z, y = expression, color = x)) +
  geom_point() +
  facet_wrap(~ gene, nrow = 2) +
  scale_color_brewer(palette = "Set1")

## ----pcaing, fig.show = 'hold'------------------------------------------------
pca_df <- as_tibble(as.data.frame(prcomp(sim$y)$x[,1:2])) %>% 
  mutate(x = factor(sim[['x']]), z = sim[['z']])

ggplot(pca_df, aes(x = PC1, y = PC2, color = x)) +
  geom_point() + scale_colour_brewer(palette = "Set1")

ggplot(pca_df, aes(x = PC1, y = PC2, color = z)) +
  geom_point()

## ----see-results, cache=TRUE--------------------------------------------------
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-6, thin = 40)
print(fit)

## ----plot-elbo----------------------------------------------------------------
plot_elbo(fit)

## ----plot-results, fig.show = 'hold', fig.width = 2.5, fig.height = 2.5-------
qplot(sim$z, trajectory(fit)) +
  xlab("True z") + ylab("Phenopath z")
qplot(sim$z, pca_df$PC1) +
  xlab("True z") + ylab("PC1")

## ----print-correlation--------------------------------------------------------
cor(sim$z, trajectory(fit))

## ----beta-df, fig.width = 6, fig.height = 3-----------------------------------
gene_names <- paste0("gene", seq_len(ncol(fit$m_beta)))
df_beta <- data_frame(beta = interaction_effects(fit),
                      beta_sd = interaction_sds(fit),
                      is_sig = significant_interactions(fit),
                      gene = gene_names)

df_beta$gene <- fct_relevel(df_beta$gene, gene_names)

ggplot(df_beta, aes(x = gene, y = beta, color = is_sig)) + 
  geom_point() +
  geom_errorbar(aes(ymin = beta - 2 * beta_sd, ymax = beta + 2 * beta_sd)) +
  theme(axis.text.x = element_text(angle = 90, hjust = 1),
        axis.title.x = element_blank()) +
  ylab(expression(beta)) +
  scale_color_brewer(palette = "Set2", name = "Significant")

## ----graph-largest-effect-size------------------------------------------------
which_largest <- which.max(df_beta$beta)

df_large <- data_frame(
  y = sim[['y']][, which_largest],
  x = factor(sim[['x']]),
  z = sim[['z']]
)

ggplot(df_large, aes(x = z, y = y, color = x)) +
  geom_point() +
  scale_color_brewer(palette = "Set1") +
  stat_smooth()

## ----construct-sceset, warning = FALSE----------------------------------------
suppressPackageStartupMessages(library(SummarizedExperiment))
exprs_mat <- t(sim$y)
pdata <- data.frame(x = sim$x)
sce <- SummarizedExperiment(assays = list(exprs = exprs_mat), 
                            colData = pdata)
sce

## ----example-using-expressionset, eval = FALSE--------------------------------
# fit <- phenopath(sce, sim$x) # 1
# fit <- phenopath(sce, "x") # 2
# fit <- phenopath(sce, ~ x) # 3

## ----initialisation-examples, eval = FALSE------------------------------------
# fit <- phenopath(sim$y, sim$x, z_init = 1) # 1, initialise to first principal component
# fit <- phenopath(sim$y, sim$x, z_init = sim$z) # 2, initialise to true values
# fit <- phenopath(sim$y, sim$x, z_init = "random") # 3, random initialisation

## ----cavi-tuning, eval = FALSE------------------------------------------------
# fit <- phenopath(sim$y, sim$x,
#                  maxiter = 1000, # 1000 iterations max
#                  elbo_tol = 1e-2, # consider model converged when change in ELBO < 0.02%
#                  thin = 20 # calculate ELBO every 20 iterations
#                  )

## ----sessioninfo--------------------------------------------------------------
sessionInfo()

N <- 100
G <- 80
G_de <- 10
G_pst <- 20
G_pst_beta <- 30
G_de_pst_beta <- 20
thin <- 5


sim <- simulate_phenopath(N, G, G_de, G_pst, G_pst_beta, G_de_pst_beta)
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-3, 
                 thin = thin, verbose = FALSE)

#' PhenoPath - genomic trajectories with heterogeneous backgrounds
#' 
#' PhenoPath learns genomic trajectories in the presence of 
#' heterogenous environmental and genetic backgrounds. It takes 
#' input gene expression measurements that are modelled
#' by a single unobserved factor (the "trajectory"). The regulation 
#' of genes along the trajectory is perturbed by an additional set of 
#' covariates (such as genetic or environmental status) allowing for 
#' the identification of covariate-trajectory interactions.
#' The model is fitted using mean-field co-ordinate ascent variational inference.
#' 
#' @param exprs_obj Input gene expression, either
#' \enumerate{
#' \item An \linkS4class{SummarizedExperiment} object, \emph{or}
#' \item A cell-by-gene matrix of normalised expression values in log form.
#' }
#' @param x The covariate vector, either
#' \enumerate{
#' \item The name of a column of \code{colData(exprs_obj)} if \code{exprs_obj} is an
#' \code{SummarizedExperiment}, \emph{or}
#' \item A numeric of factor vector of length equal to the 
#' number of cells, \emph{or}
#' \item A formula from which to build a model matrix from \code{colData(exprs_obj)}, 
#' if \code{exprs_obj} is a \linkS4class{SummarizedExperiment}
#' }
#' @param sce_assay The assay from \code{exprs_obj} to use as expression if
#' \code{exprs_object} is a \code{SummarizedExperiment}
#' @param elbo_tol The relative pct change in the ELBO below 
#' which is considered converged.
#' See convergence section in details below.
#' @param z_init The initialisation of the latent trajectory. Should be one of
#' \enumerate{
#' \item A positive integer describing which principal component of the data should
#' be used for initialisation (default 1), \emph{or}
#' \item A numeric vector of length number of samples to be used 
#' directly for initialisation, \emph{or}
#' \item The text character \code{"random"}, for random initialisation 
#' from a standard normal distribution.
#' }
#' @param ... Additional arguments to be passed to \code{\link{clvm}}. 
#' See description below for more details or call \code{?clvm}.
#' 
#' @return An S3 structure with the following entries:
#' \itemize{
#' \item \code{m_z} The converged mean estimates of the trajectory
#' \item \code{s_z} The converged standard deviation estimates of z
#' \item \code{m_beta} A P-by-G matrix of interaction coefficients
#' \item \code{s_beta} A P-by-G matrix of interaction standard deviations
#' }
#' 
#' @details
#' \strong{Input expression}
#' 
#' If an \code{SummarizedExperiment} is provided, \code{assay(exprs_obj, sce_assay)} 
#' is used. 
#' This is assumed to be in
#' a form that is suitably normalised and approximately normal, such as 
#' the log of TPM values (plus a suitable offset) or 
#' similar.
#'
#' \strong{Encoding covariates}
#' 
#' See the vignette.
#' 
#' \strong{Convergence of variational inference}
#' 
#' It is strongly recommended to call \code{plot_elbo(phenopath_fit)} after the fitting procedure to
#' ensure the ELBO has approximately converged (though convergence metrics are printed during the
#' fitting process). For a good introduction to variational inference see Blei, D.M., Kucukelbir, A. & McAuliffe, J.D., 2017. Variational Inference: A Review for Statisticians. Journal of the American Statistical Association.
#'
#' \strong{Additional arguments}
#' 
#' Addition arguments to \code{clvm} are passed via \code{...}. For full documentation, call \code{?clvm}.
#' Some notable options:
#' \itemize{
#' \item \code{thin} - The ELBO is expensive to compute for larger datasets. The model will
#' compute the ELBO and compare convergence every \code{thin} iterations.
#' \item \code{q} and \code{tau_q} - Priors (such as capture times) for the latent space. Note that
#' \code{model_mu} should be true if \code{q} is non-zero.
#' \item \code{scale_y} By default the input expression is centre-scaled for each gene. If \code{scale_y}
#' is \code{FALSE} this does not happen - but note that \code{model_mu} should be \code{TRUE} in such a case.
#' }
#' 
#' @seealso \code{\link{clvm}} for the underlying CAVI function, \code{\link{trajectory}}
#' to extract the latent trajectory, \code{\link{interaction_effects}} for the interaction effect
#' sizes, \code{\link{significant_interactions}} for the results of Bayesian significance testing.
#' 
#' @examples
#' sim <- simulate_phenopath() # returns a list with gene expression in y and covariates in x
#' fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
#' 
#' # Extract the trajectory
#' z <- trajectory(fit)
#' 
#' @importFrom methods is
#' @importFrom stats model.matrix
#' @importFrom SummarizedExperiment assay colData
#' @export
phenopath <- function(exprs_obj, x, sce_assay = "exprs",
                      elbo_tol = 1e-5, z_init = 1, ...) {
  
  is_eset <- is_matrix <- FALSE
  N <- -1 # Number of samples
  y <- NULL # Expression object
  xx <- NULL # Covariate matrix we'll actually use
  
  # Get expression input
  if(is(exprs_obj, "SummarizedExperiment")) {
    is_eset <- TRUE
    N <- ncol(exprs_obj)
    y <- t(assay(exprs_obj, sce_assay))
    
  } else if(is.matrix(exprs_obj) && is.numeric(exprs_obj)) {
    is_matrix <- TRUE
    N <- nrow(exprs_obj)
    y <- exprs_obj
  } else {
    stop("Input exprs_obj must either be an ExpressionSet or numeric matrix of gene expression")
  }
  
  # Sort covariate input
  if(length(x) == 1 && is.character(x)) {
    # x describes a column of pData(exprs_obj)
    if(!is_eset || (!x %in% names(colData(exprs_obj)))) {
      stop("If x is a character then exprs_obj must be an ExpressionSet (where x represents a column in colData(exprs_obj))")
    }
    xx <- colData(exprs_obj)[[x]]
  } else if(is.vector(x) | is.factor(x)) {
    if(length(x) != N) {
      stop("If x is a character vector or factor it must be of compatible dimensions to exprs_obj (same number of samples)")
    }
    xx <- x
  } else if(is(x, "formula")) {
    xx <- x
  } else if(is.matrix(x)) {
    stopifnot(nrow(x) == N)
    x_mat <- xx <- x
  }
  
  # Couple of extra checks
  if(is.character(xx)) {
    warning("x is a character vector...coercing to factors...")
    xx <- factor(xx)
  }
  if(!is.factor(xx) && !is.numeric(xx) && !is(xx, "formula")) {
    stop("If x is a vector it must be of type numeric, factor, or character")
  }

  # If single vector then scale
  if(is.numeric(xx) && !is.matrix(xx)) xx <- scale_vec(xx)
  
  # Now we come to the case of factors
  if(is.factor(xx)) {
    x_mat <- model.matrix(~ xx, data.frame(xx))
    # COME BACK AND CHANGE ME
    x_mat <- x_mat[,-1, drop = FALSE] # Remove intercept
    x_mat <- apply(x_mat, 2, scale_vec) # Centre scale values
  } else if(is(xx, "formula")) {
    if(!is(exprs_obj, "SummarizedExperiment")) {
      stop("If x is a formula, y must be an SummarizedExperiment")
    }
    x_mat <- model.matrix(xx, colData(exprs_obj))
    x_mat <- x_mat[,-1, drop = FALSE] # Remove intercept
    x_mat <- apply(x_mat, 2, scale_vec) # Centre scale values
  } else if(is.vector(xx)) {
    x_mat <- matrix(xx)
  }
  
  cl_fit <- clvm(y, x_mat, elbo_tol = elbo_tol, z_init = z_init, ...)
  
  cl_fit$N <- nrow(y)
  cl_fit$G <- ncol(y)
  
  feature_names <- colnames(y)
  if(is.null(feature_names)) feature_names <- paste0("feature_", seq_len(cl_fit$G))
  cl_fit$feature_names <- feature_names
  
  return(
    structure(cl_fit, class = "phenopath_fit")
  )
}

#' Print a PhenoPath fit
#' 
#' @param x A \code{phenopath_fit} returned by a call to \code{phenopath}
#' @param ... Additional arguments
#' 
#' @export
#' 
#' @return A string representation of a \code{phenopath_fit} object.
#' 
#' @examples 
#' sim <- simulate_phenopath() 
#' fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
#' print(fit)
print.phenopath_fit <- function(x, ...) {
  msg <- paste("PhenoPath fit with",
               x$N, "cells and", x$G, "genes")
  cat(msg)
}

#' Get the latent PhenoPath trajectory
#' 
#' @param phenopath_fit An object of class \code{phenopath_fit}
#' @return A vector of latent trajectory (pseudotime) values
#' 
#' @examples 
#' sim <- simulate_phenopath() 
#' fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
#' z <- trajectory(fit)
#' 
#' @export
trajectory <- function(phenopath_fit) {
  stopifnot(is(phenopath_fit, "phenopath_fit"))
  return(phenopath_fit$m_z)
}

#' Get the interaction effect sizes
#' 
#' @param phenopath_fit An object of class \code{phenopath_fit}
#' @return TODO
#' 
#' @export
#' 
#' @examples 
#' sim <- simulate_phenopath() 
#' fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
#' beta <- interaction_effects(fit)
interaction_effects <- function(phenopath_fit) {
  stopifnot(is(phenopath_fit, "phenopath_fit"))
  P <- nrow(phenopath_fit$m_beta)
  return(phenopath_fit$m_beta[1:P,,drop=TRUE])
}

#' Get the interaction standard deviations
#' 
#' @param phenopath_fit An object of class \code{phenopath_fit}
#' @return TODO
#' 
#' @export
#' 
#' @examples 
#' sim <- simulate_phenopath() 
#' fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
#' beta_sd <- interaction_sds(fit)
interaction_sds <- function(phenopath_fit) {
  stopifnot(is(phenopath_fit, "phenopath_fit"))
  P <- nrow(phenopath_fit$m_beta)
  return(sqrt(phenopath_fit$s_beta[1:P,,drop=TRUE]))
}

#' Significance testing for interaction features
#' 
#' Given the results of \code{clvm}, decide which features show significant
#' iteractions between the latent trajectory and covariates. Significant 
#' features are designated as those where the variational mean of the interaction
#' coefficient falls outside the \eqn{n \sigma} interval of 0.
#' 
#' @param phenopath_fit The results of a call to \code{clvm}
#' @param n The number of standard deviations away from 0 the posterior
#' estimate of beta should be to be designated significant.
#' 
#' @return A logical vector describing whether each feature 
#' passes the significance test.
#' 
#' @export
#' @examples 
#' sim <- simulate_phenopath() 
#' fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
#' signints <- significant_interactions(fit)
significant_interactions <- function(phenopath_fit, n = 3) {
  stopifnot(is(phenopath_fit, "phenopath_fit"))
  m_beta <- phenopath_fit$m_beta
  pos_sd <- sqrt(phenopath_fit$s_beta)
  
  sig <- sapply(seq_len(nrow(m_beta)), function(i) {
    m_beta[i,] - n * pos_sd[i,] > 0 | m_beta[i,] + n * pos_sd[i,] < 0
  })
  if(ncol(sig) == 1) sig <- as.vector(sig)
  return(sig)
}


#' Tidy summary of interactions
#' 
#' Computes a tidy data frame of the interaction effects, pathway scores, and significance for
#' each gene and covariate interaction.
#' 
#' @param phenopath_fit An object returned by a call to \code{phenopath}
#' @param n The number of standard deviations away from 0 the posterior of the 
#' interaction effect sizes should be to be designated as significant
#' 
#' @importFrom tidyr gather
#' @export
#' 
#' @return A data frame with the following columns:
#' \itemize{
#' \item \code{feature} The feature (usually gene)
#' \item \code{covariate} The covariate, specified from the order originally supplied to 
#' the call to \code{phenopath}
#' \item \code{interaction_effect_size} The effect size of the interaction (\eqn{beta} 
#' from the statistical model)
#' \item \code{significant} Boolean for whether the interaction effect is significantly
#' different from 0
#' \item \code{chi} The precision of the ARD prior on \eqn{beta}
#' \item \code{pathway_loading} The pathway loading \eqn{lambda}, showing the overall
#' effect for each gene marginalised over the covariate
#' }
#' @examples 
#' sim <- simulate_phenopath() 
#' fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
#' interactions(fit)
interactions <- function(phenopath_fit, n = 3) {
  covariate <- interaction_effect_size <- feature <- significant_interaction <- NULL
  
  ie <- interaction_effects(phenopath_fit)
  sig <- significant_interactions(phenopath_fit)
  if(is.vector(ie)) {
    ie <- matrix(ie)
  }
  if(is.vector(sig)) {
    sig <- matrix(sig)
  }
  
  chi <- dplyr::as_data_frame(t(phenopath_fit$chi_exp))
  
  ie <- dplyr::as_data_frame(ie)
  sig <- dplyr::as_data_frame(sig)
  names(ie) <- names(sig) <- names(chi) <- paste0("covariate_", 1:ncol(ie))
  ie$feature <- sig$feature <- chi$feature <- phenopath_fit$feature_names
  
  ie_tidy <- gather(ie, covariate, interaction_effect_size, -feature)  
  sig_tidy <- gather(sig, covariate, significant_interaction, -feature)  
  chi_tidy <- gather(chi, covariate, chi, -feature)
  
  interaction_df <- dplyr::inner_join(ie_tidy, sig_tidy, by = c("feature", "covariate"))
  interaction_df <- dplyr::inner_join(interaction_df, chi_tidy, by = c("feature", "covariate"))
  
  lambda_df <- dplyr::data_frame(pathway_loading = phenopath_fit$m_lambda,
                          feature = phenopath_fit$feature_names)
  
  interaction_df <- dplyr::inner_join(interaction_df, lambda_df, by = "feature")
  interaction_df
}

context("Simulations")

test_that("simulate_phenopath() returns valid object", {
  ## Basic return structure
  expect_is(sim, "list")
  expect_equal(length(sim), 4)
  expect_identical(names(sim), c("parameters", "y", "x", "z"))
  
  ## Check everything correct size
  expect_equal(nrow(sim$y), N)
  expect_equal(ncol(sim$y), G)
  expect_equal(length(sim$x), N)
  expect_equal(length(sim$z), N)
})

test_that("simulate_phenopath() returns correct parameter specs", {
  param_df <- sim$parameters
  expect_equal(nrow(param_df), G)
  expect_identical(names(param_df), c("alpha", "lambda", "beta", "regime"))  
  
  regime_table <- table(param_df$regime)
  expect_equal(regime_table[['de']], G_de)
  expect_equal(regime_table[['pst']], G_pst)
  expect_equal(regime_table[['pst_beta']], G_pst_beta)
  expect_equal(regime_table[['de_pst_beta']], G_de_pst_beta)
})



context("phenopath() function")

test_that("phenopath() returns valid object", {
  expect_is(fit, "phenopath_fit")
  expect_equal(fit$thin, thin)
  expect_equal(fit$N, N)
  expect_equal(fit$G, G)
})
  
test_that("CAVI for CLVM has correctly sized outputs", {
  expect_equal(length(fit$m_z), N)
  expect_equal(length(fit$m_lambda), G)
  expect_equal(length(fit$m_mu), G)
  expect_equal(length(fit$a_tau), G)
  expect_equal(length(fit$b_tau), G)
  
  exp_dims <- c(1, G)
  expect_equal(dim(fit$chi_exp), exp_dims)
  expect_equal(dim(fit$m_alpha), exp_dims)
  expect_equal(dim(fit$m_beta), exp_dims)
})


test_that("phenopath() accepts SummarizedExperiments", {
  library(SummarizedExperiment)
  exprs_mat <- t(sim$y)
  pdata <- data.frame(x = sim$x)
  sce <- SummarizedExperiment(assays = list(exprs = exprs_mat), 
                              colData = pdata)
  
  ## We'll test phenopath with three different x inputs:
  ## (1) Character vector from pData(sce)
  ## (2) Formula
  ## (3) The values themselves
  ## and check we get the same result for all
  
  set.seed(123)
  suppressWarnings(fit1 <- phenopath(sce, "x", maxiter = 4, verbose = FALSE))
  set.seed(123)
  suppressWarnings(fit2 <- phenopath(sce, ~ x, maxiter = 4, verbose = FALSE))
  set.seed(123)
  suppressWarnings(fit3 <- phenopath(sce, colData(sce)$x, maxiter = 4, verbose = FALSE))
  
  elbos <- sapply(list(fit1$elbos, fit2$elbos, fit3$elbos), tail, n = 1)
  expect_equal(length(unique(elbos)), 1)
})

#' Plots the ELBO
#' 
#' Plots the evidence lower bound (ELBO) as a function of iterations
#' @export
#' @param fit An object returned by a call to \code{phenopath}
#' @import ggplot2
#' @examples 
#' sim <- simulate_phenopath()
#' fit <- phenopath(sim$y, sim$x)
#' plot_elbo(fit)
#' @return A \code{ggplot2} object of the ELBO against the number of iterations
plot_elbo <- function(fit) {
  stopifnot(is(fit, "phenopath_fit"))
  elbo <- fit$elbos[-(1)]
  ggplot2::qplot(fit$thin * seq_along(elbo) + 1, elbo, geom = c("point", "line")) + xlab("Iter") + ylab("ELBO")
}

#' Simulate one gene
#' 
#' Simulate one gene from the model given paramters, z and covariates
#' 
#' @keywords internal
#' @return A length-N gene expression vector simulated with the 
#' PhenoPath mean function for the given parameters
simulate_one_gene <- function(N, pst, x, alpha = 0, c = 0, beta = 0, tau = 1e6) {
  mu <- alpha * x + (c + beta * x) * pst
  rnorm(N, mu, 1 / sqrt(tau))
}

#' Sample parameters for simulation
#' 
#' Sample parameters from de regime
#' @keywords internal
#' @name sample_fns
#' 
#' @return A length-3 vector of parameters corresponding to the particular 
#' simulation regime
sample_de <- function() {
  alpha <- sample(c(-1, 1), 1)
  c <- beta <- 0
  return(c(alpha, c, beta))
}

#' Sample parameters from pseudotime regime
#' @keywords internal
#' @name sample_fns
sample_pst <- function() {
  c <- 1 * sample(c(-1, 1), 1)
  # c <- rnorm(1)
  alpha <- beta <- 0
  return(c(alpha, c, beta))
}

#' Sample parameters from pseudotime-interaction regime
#' @keywords internal
#' @name sample_fns
sample_pst_beta <- function() {
  c <- 1 * sample(c(-1, 1), 1)  
  # c <- rnorm(1)
  beta <- 1 * sample(c(-1, 1), 1)  
  alpha <- 0
  return(c(alpha, c, beta))
}

#' Sample paramters from de-pseudotime-interaction regime
#' @keywords internal
#' @name sample_fns
sample_de_pst_beta <- function() {
  c <- 1 * sample(c(-1, 1), 1)  
  # c <- rnorm(1)
  beta <- 1 * sample(c(-1, 1), 1)  
  alpha <- sample(c(-1, 1), 1)
  return(c(alpha, c, beta))
}


#' Simulate from phenopath model
#' 
#' Simulate synthetic data from the phenopath model, where each gene is sampled from 
#' one of four types  (see details).
#' 
#' @param N Number of samples to simulate
#' @param G Number of genes to simulate. Should be divisible by 4
#' @param G_de Number of genes to simulate from the \emph{differential expression} regime
#' @param G_pst Number of genes to simulate from the \emph{pseudotime} regime
#' @param G_pst_beta Number of genes to simulate from the \emph{pseudotime + beta interactions} regime
#' @param G_de_pst_beta Number of genes to simulate from the \emph{differential expression + pseudotime + interactions} regime
#' 
#' @return A list with four entries:
#' \itemize{
#' \item \code{parameters} A \code{tibble} with an entry for each gene and a column
#' for each parameter value and simulation regime (see details).
#' \item \code{y} The N-by-G simulated gene expression matrix.
#' \item \code{x} The N-length vector of covariates.
#' \item \code{z} The N-length vector of pseudotimes.
#' }
#' 
#' 
#' @details
#' 
#' Will simulate data for a number of genes corresponding to one of four regimes:
#' \enumerate{
#' \item \code{de} ('differential expression'), where the gene has no association to the latent 
#' trajectory and exhibits differential expression only
#' \item \code{pst} ('pseudotime'), the gene shows pseudotemporal regulation but no 
#' differential regulation
#' \item \code{pst_beta} ('pseudotime + beta interactions'), the gene shows pseudotemporal regulation
#' that is modulated by covariate interactions 
#' \item \code{de_pst_beta} ('differential expression + pseudotime + interactions), all of the above
#' }
#' 
#' @export
#' @importFrom dplyr mutate
#' @importFrom tibble as_data_frame
#' 
#' @examples
#' sim <- simulate_phenopath()
simulate_phenopath <- function(N = 100, G = 40,
                               G_de = NULL, G_pst = NULL,
                               G_pst_beta = NULL, G_de_pst_beta = NULL) {

  setifnotnull <- function(x) ifelse(is.null(x), G / 4, x)
  if(any(is.null(c(G_de, G_pst, G_pst_beta, G_de_pst_beta))) && (G %% 4 != 0)) {
    stop("If any of G_de, G_pst, G_pst_beta, G_de_pst_beta are left NULL then G must be divisible by 4 four default")
  }
  if(all(!is.null(c(G_de, G_pst, G_pst_beta, G_de_pst_beta)))) {
    G <- G_de + G_pst + G_pst_beta + G_de_pst_beta
  }
  G_de <- setifnotnull(G_de)
  G_pst <- setifnotnull(G_pst)
  G_pst_beta <- setifnotnull(G_pst_beta)
  G_de_pst_beta <- setifnotnull(G_de_pst_beta)
  
  
  x <- rep(c(1, -1), each = N / 2)
  pst <- rnorm(N)
  pst <- (pst - mean(pst)) / sd(pst)

  de_pars <- pst_pars <- pst_beta_pars <- de_pst_beta_pars <- NULL
  if(G_de > 0)
    de_pars <- t(replicate(G_de, sample_de()))
  if(G_pst > 0)
    pst_pars <- t(replicate(G_pst, sample_pst()))
  if(G_pst_beta > 0)
    pst_beta_pars <- t(replicate(G_pst_beta, sample_pst_beta()))
  if(G_de_pst_beta > 0)
    de_pst_beta_pars <- t(replicate(G_de_pst_beta, sample_de_pst_beta()))
  
  regimes <- c("de", "pst", "pst_beta", "de_pst_beta")
  
  par_list <- list(de_pars, pst_pars, pst_beta_pars, de_pst_beta_pars)
  par_list <- par_list[!sapply(par_list, is.null)]
  pars <- as_data_frame(do.call("rbind", par_list))
  names(pars) <- c("alpha", "lambda", "beta")
  pars <- mutate(pars, regime = rep(regimes, times = c(G_de, G_pst, G_pst_beta, G_de_pst_beta)))
  
  #pars$alpha <- scale_vec(pars$alpha)
  #pars$lambda <- scale_vec(pars$lambda)
  #pars$beta <- scale_vec(pars$beta)
  
  gex <- apply(pars, 1, function(p) {
    p <- as.numeric(p[1:3])
    simulate_one_gene(N, pst, x, p[1], p[2], p[3])
  })
  
  return(list(parameters = pars, y = gex, x = x, z = pst))
}

library(Rcpp)


#' Fit a CLVM Model
#' 
#' Fit a covariate latent variable model using coordinate ascent
#' variational inference.
#' 
#' @param y A N-by-G (dynamic) input matrix 
#' @param x A N-by-P (static) input matrix
#' @param maxiter Maximum number of CAVI iterations
#' @param elbo_tol The (percent) change in the ELBO below which it is 
#' considered converged
#' @param thin The number of iterations to wait each time before
#' re-calculating the elbo
#' @param verbose Print convergence messages
#' @param tau_q Hyperparameter tau_q
#' @param tau_mu Hyperparameter tau_mu
#' @param tau_c Hyperparameter tau_c
#' @param a Hyperparameter a
#' @param b Hyperparameter b
#' @param tau_alpha Hyperparameter tau_alpha
#' @param a_beta Hyperparameter a_beta
#' @param b_beta Hyperparameter b_beta
#' @param q Priors on the latent variables
#' @param model_mu Logical - should a gene-specific intercept term be modelled?
#' @param scale_y Logical - should the expression matrix be centre scaled?
#' @param z_init The initialisation of the latent trajectory. Should be one of
#' \enumerate{
#' \item A positive integer describing which principal component of the data 
#' should be used for initialisation (default 1), \emph{or}
#' \item A numeric vector of length number of samples to be used directly 
#' for initialisation, \emph{or}
#' \item The text character \code{"random"}, for random initialisation from 
#' a standard normal distribution.
#' }
#' 
#' @import Rcpp
#' @return 
#' A list whose entries correspond to the converged values of the
#' variational parameters along with the ELBO.
#' 
#' @importFrom stats coef lm prcomp
#' @importFrom stats rnorm sd
#' 
#' @useDynLib phenopath
#' 
#' @examples 
#' sim <- simulate_phenopath()
#' fit <- clvm(sim$y, matrix(sim$x))
#' 
#' 
#' @export
clvm <- function(y, x, maxiter = 1e4,
                 elbo_tol = 1e-5,
                 thin = 1,
                 verbose = TRUE,
                 z_init = 1,
                 tau_q = 1,
                 tau_mu = 1,
                 tau_c = 1,
                 a = 2, b = 2,
                 tau_alpha = 1,
                 a_beta = 10, b_beta = 1,
                 q = rep(0, nrow(y)),
                 model_mu = FALSE,
                 scale_y = TRUE) {

  N <- nrow(y)
  G <- ncol(y)
  P <- ncol(x)
  
  if(scale_y) {
    y <- scale(y)
  }
  
  ## Parameter initialisation
  
  a_tau <- rep(1, G) # rgamma(G, 2)
  b_tau <- rep(1, G) # rgamma(G, 2)


  if(P == 1) {
    f <- apply(y, 2, function(yy) {
      fit <- lm(yy ~ 0 + x[,1])
      coef(fit)
    })
    m_alpha <- matrix(f, nrow = 1, ncol = G)
  } else {
    m_alpha <- matrix(0, nrow = P, ncol = G)
  }
  s_alpha <- matrix(0.1, nrow = P, ncol = G)

  m_beta <- matrix(0, nrow = P, ncol = G)
  s_beta <- matrix(0.1, nrow = P, ncol = G)

  a_chi <- matrix(0.1, nrow = P, ncol = G)
  b_chi <- matrix(0.01, nrow = P, ncol = G)

  ## Three options for pst init:
  ## (1) A numeric vector of initial values
  ## (2) A single integer specifying which PC to initialise to
  ## (3) A text character "random" in which case they're drawn N(0,1)

  if(length(z_init) == 1 && is.numeric(z_init)) {  
    m_z <- prcomp(scale(y))$x[,z_init]
    m_z <- (m_z - mean(m_z)) / sd(m_z)
  } else if(length(z_init) == N && is.numeric(z_init)) {
    m_z <- z_init
  } else if(z_init == "random") {
    m_z <- rnorm(N)
  } else if(z_init == "pc1_with_noise") {
    m_z <- prcomp(scale(y))$x[,1]
    m_z <- (m_z - mean(m_z)) / sd(m_z)
    m_z <- rnorm(N, m_z)
  } else {
    stop("z initialisation not recognised")
  }
  
  s_z <- rep(0.1, N)

  m_lambda <- apply(y, 2, function(yy) coef(lm(yy ~ m_z))[2])
  m_lambda <- (m_lambda - mean(m_lambda)) / sd(m_lambda)
  s_lambda <- rep(0.1, G)

  m_mu <- rep(0, G)
  if(model_mu) {
    s_mu <- rep(0.1, G)
  } else {
    s_mu <- rep(0, G)
  }
  
  
  if(verbose) {
    cat("Iteration\tELBO\tChange (%) \n")
  }

  elbo_old <- -Inf
  delta_elbo <- Inf
  elbos <- NULL
  elys <- elps <- elqs <- NULL
  ts <- cs <- betas <- taus <- chis <- alphas <- betas <- NULL
  i <- 1

  while(i < maxiter & delta_elbo > elbo_tol) {
    
    if(model_mu) {
      cumu <- cavi_update_mu(y, x, m_z, m_lambda, m_alpha, m_beta, 
                             a_tau, b_tau, tau_mu)
      m_mu <- cumu[,1]; s_mu <- cumu[,2]
    } else {
      m_mu <- rep(0,G)
      s_mu <- rep(0,G)
    }
    
    # Update lambda  
    culam <- cavi_update_lambda(y, x, m_z, s_z, m_alpha, 
                                m_beta, a_tau, b_tau,
                                m_mu, tau_c)
    m_lambda <- culam[,1]; s_lambda <- culam[,2]

    # Update tau
    cut <- cavi_update_tau(y, x, m_z, s_z, m_lambda, s_lambda, m_alpha, 
                           m_beta, s_alpha, s_beta, m_mu, s_mu, a, b)
    a_tau <- cut[,1]; b_tau <- cut[,2]

    alpha_sum <- calculate_greek_sum(m_alpha, x)
    beta_sum <- calculate_greek_sum(m_beta, x)
    
    for(g in 1:G) {
      for(p in 1:P) {
        ## First calculate alpha update
        cua <- cavi_update_alpha(beta_sum, p-1, g-1, y, x, m_z, 
                                 m_lambda, m_alpha, m_beta, a_tau, b_tau,
                                 m_mu, tau_alpha)
        alpha_sum <- update_greek_sum(g-1, p-1, alpha_sum, 
                                      m_alpha[p,g], cua[1], x)
        m_alpha[p,g] <- cua[1] 
        s_alpha[p,g] <- cua[2]
        
        ## Calculate beta update
        cub <- cavi_update_beta(alpha_sum, p-1, g-1, y, x, m_z, 
                                s_z, m_lambda, m_alpha, m_beta, a_tau,
                                b_tau, a_chi, b_chi, m_mu)
        
        beta_sum <- update_greek_sum(g-1, p-1, beta_sum, m_beta[p,g], cub[1], x)
        m_beta[p,g] <- cub[1]
        s_beta[p,g] <- cub[2]

        # Calculate chi update
        cuch <- cavi_update_chi(m_beta[p,g], s_beta[p,g], a_beta, b_beta)
        a_chi[p,g] <- cuch[1]; b_chi[p,g] <- cuch[2] 
      }
    }

    cuz <- cavi_update_z(y, x, m_lambda, m_mu, s_lambda, 
                         m_alpha, m_beta, s_beta, a_tau, b_tau, q, tau_q)
    m_z <- cuz[,1]; s_z <- cuz[,2]

    ## calculate elbo and report
    if(i %% thin == 0) {
      elbo_vec <- calculate_elbo(y, x, m_z, s_z, m_lambda, s_lambda, 
                                 m_alpha, s_alpha, m_beta, 
                                 s_beta, a_tau, b_tau, a_chi, 
                                 b_chi, m_mu, s_mu, q, tau_q, 
                                 tau_mu, tau_c, a, b, tau_alpha, 
                                 a_beta, b_beta, as.integer(model_mu)) 
      
      elbo <- elbo_vec[1] + elbo_vec[2] - elbo_vec[3]

      delta_elbo <- abs((elbo - elbo_old) / elbo * 100 / thin)
      if(verbose) {
        cat(paste("[", i, "] \t", elbo, "\t", delta_elbo), "\n")
      }
      elbo_old <- elbo
      elbos <- c(elbos, elbo)
    }
    i <- i + 1
  }
  
  if(i == maxiter) {
    warning("ELBO not converged")
  }

  rlist <- list(m_z = m_z, s_z = s_z, m_lambda = m_lambda, 
                s_lambda = s_lambda, m_mu = m_mu, m_alpha = m_alpha,
                s_alpha = s_alpha, a_tau = a_tau, b_tau = b_tau,
                m_beta = m_beta, s_beta = s_beta, chi_exp = a_chi / b_chi,
                elbos = elbos, thin = thin)
  return(rlist)
}
  



#' Scale a vector to have mean 0 and variance 1
#' 
#' Scales vector to mean 0 variance 1 unless input standard deviation is 0
#' in which case original vector is returned
#' @keywords internal
#' @param x Input vector to scale
#' @return Scaled vector
scale_vec <- function(x) {
  std_dev <- sd(x)
  if(std_dev != 0) {
    return( (x - mean(x)) / std_dev )
  } else {
    return(x)
  }
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

calculate_greek_sum <- function(greek, x) {
    .Call('_phenopath_calculate_greek_sum', PACKAGE = 'phenopath', greek, x)
}

update_greek_sum <- function(g, p, greek_sum, old_greek, new_greek, x) {
    .Call('_phenopath_update_greek_sum', PACKAGE = 'phenopath', g, p, greek_sum, old_greek, new_greek, x)
}

greek_square_exp <- function(m_g, s_g, x) {
    .Call('_phenopath_greek_square_exp', PACKAGE = 'phenopath', m_g, s_g, x)
}

calculate_fg <- function(g, y, m_z, s_z, m_lambda, s_lambda, m_mu, s_mu, alpha_sum, beta_sum, alpha_square_sum, beta_square_sum) {
    .Call('_phenopath_calculate_fg', PACKAGE = 'phenopath', g, y, m_z, s_z, m_lambda, s_lambda, m_mu, s_mu, alpha_sum, beta_sum, alpha_square_sum, beta_square_sum)
}

cavi_update_z <- function(y, x, m_lambda, m_mu, s_lambda, m_alpha, m_beta, s_beta, a_tau, b_tau, q, tau_q) {
    .Call('_phenopath_cavi_update_z', PACKAGE = 'phenopath', y, x, m_lambda, m_mu, s_lambda, m_alpha, m_beta, s_beta, a_tau, b_tau, q, tau_q)
}

cavi_update_mu <- function(y, x, m_z, m_lambda, m_alpha, m_beta, a_tau, b_tau, tau_mu) {
    .Call('_phenopath_cavi_update_mu', PACKAGE = 'phenopath', y, x, m_z, m_lambda, m_alpha, m_beta, a_tau, b_tau, tau_mu)
}

cavi_update_lambda <- function(y, x, m_z, s_z, m_alpha, m_beta, a_tau, b_tau, m_mu, tau_c) {
    .Call('_phenopath_cavi_update_lambda', PACKAGE = 'phenopath', y, x, m_z, s_z, m_alpha, m_beta, a_tau, b_tau, m_mu, tau_c)
}

cavi_update_tau <- function(y, x, m_z, s_z, m_lambda, s_lambda, m_alpha, m_beta, s_alpha, s_beta, m_mu, s_mu, a, b) {
    .Call('_phenopath_cavi_update_tau', PACKAGE = 'phenopath', y, x, m_z, s_z, m_lambda, s_lambda, m_alpha, m_beta, s_alpha, s_beta, m_mu, s_mu, a, b)
}

cavi_update_alpha <- function(beta_sum, p, g, y, x, m_z, m_lambda, m_alpha, m_beta, a_tau, b_tau, m_mu, tau_alpha) {
    .Call('_phenopath_cavi_update_alpha', PACKAGE = 'phenopath', beta_sum, p, g, y, x, m_z, m_lambda, m_alpha, m_beta, a_tau, b_tau, m_mu, tau_alpha)
}

cavi_update_beta <- function(alpha_sum, p, g, y, x, m_z, s_z, m_lambda, m_alpha, m_beta, a_tau, b_tau, a_chi, b_chi, m_mu) {
    .Call('_phenopath_cavi_update_beta', PACKAGE = 'phenopath', alpha_sum, p, g, y, x, m_z, s_z, m_lambda, m_alpha, m_beta, a_tau, b_tau, a_chi, b_chi, m_mu)
}

cavi_update_chi <- function(m_beta_pg, s_beta_pg, a_beta, b_beta) {
    .Call('_phenopath_cavi_update_chi', PACKAGE = 'phenopath', m_beta_pg, s_beta_pg, a_beta, b_beta)
}

calculate_E_log_Y_given_theta <- function(y, x, m_z, s_z, m_lambda, s_lambda, m_alpha, s_alpha, m_beta, s_beta, a_tau, b_tau, m_mu, s_mu) {
    .Call('_phenopath_calculate_E_log_Y_given_theta', PACKAGE = 'phenopath', y, x, m_z, s_z, m_lambda, s_lambda, m_alpha, s_alpha, m_beta, s_beta, a_tau, b_tau, m_mu, s_mu)
}

calculate_E_log_p <- function(m_z, s_z, m_lambda, s_lambda, m_alpha, s_alpha, m_beta, s_beta, a_tau, b_tau, m_mu, s_mu, a_chi, b_chi, q, tau_q, tau_mu, tau_c, a, b, tau_alpha, a_beta, b_beta) {
    .Call('_phenopath_calculate_E_log_p', PACKAGE = 'phenopath', m_z, s_z, m_lambda, s_lambda, m_alpha, s_alpha, m_beta, s_beta, a_tau, b_tau, m_mu, s_mu, a_chi, b_chi, q, tau_q, tau_mu, tau_c, a, b, tau_alpha, a_beta, b_beta)
}

calculate_E_log_q <- function(s_z, s_lambda, s_alpha, s_beta, a_tau, b_tau, s_mu, a_chi, b_chi, model_mu) {
    .Call('_phenopath_calculate_E_log_q', PACKAGE = 'phenopath', s_z, s_lambda, s_alpha, s_beta, a_tau, b_tau, s_mu, a_chi, b_chi, model_mu)
}

calculate_elbo <- function(y, x, m_z, s_z, m_lambda, s_lambda, m_alpha, s_alpha, m_beta, s_beta, a_tau, b_tau, a_chi, b_chi, m_mu, s_mu, q, tau_q, tau_mu, tau_c, a, b, tau_alpha, a_beta, b_beta, model_mu) {
    .Call('_phenopath_calculate_elbo', PACKAGE = 'phenopath', y, x, m_z, s_z, m_lambda, s_lambda, m_alpha, s_alpha, m_beta, s_beta, a_tau, b_tau, a_chi, b_chi, m_mu, s_mu, q, tau_q, tau_mu, tau_c, a, b, tau_alpha, a_beta, b_beta, model_mu)
}

---
title: "Introduction to PhenoPath"
author: "Kieran R Campbell"
date: "January 2019"
output: BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{Vignette Title}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: bibliography.bib
---

```{r load-packages, include=FALSE}
knitr::opts_chunk$set(fig.width = 5, fig.height = 3)

suppressPackageStartupMessages({
  library(dplyr)
  library(dplyr)
  library(ggplot2)
  library(tidyr)
  library(forcats)
  library(phenopath)
})
```

# Overview of PhenoPath

## The PhenoPath model

PhenoPath models gene expression expression $y$ in terms of a latent pathway score (pseudotime) $z$. Uniquely, the evolution of genes along the trajectory isn't common to each gene but can be perturbed by an additional sample-specific covariate $\beta$. For example, this could be the mutational status of each sample or a drug that each sample was exposed to.

<img src="phenopath_model.png" width="500"/>

This software infers both the latent pathway scores $z_n$ and interaction coefficients $\beta_{pg}$ for samples $n = 1, \ldots, N$, covariates $p = 1, \ldots, P$ and genes $G = 1, \ldots, G$.

## Mean-field variational inference

Inference is performed using co-ordinate ascent mean field variational inference. This attempts to find a set of approximating distributions $q(\mathbf{\theta}) = \prod_i q_i(\theta_i)$ for each variable $i$ by minimising the KL divergence between the KL divergence between the variational distribution and the true posterior. For a good overview of variational inference see @blei2016variational.

# Example on simulated data

## Simulating data

We can simulate data according to the PhenoPath model via a call to `simulate_phenopath()`:

```{r simulate-data}
set.seed(123L)
sim <- simulate_phenopath()
```

This returns a list with four entries:

```{r sim-structure}
print(str(sim))
```

* `parameters` is a data frame with the simulated parameters, with a column for each of the parameters $\alpha$, $\beta$ and $\lambda$, and a row for each gene. There is an additional column specifying from which regime that gene has been simulated (see `?simulate_phenopath` for details).
* `y` is the $N \times G$ matrix of gene expression
* `x` is the $N$-length vector of covariates
* `z` is the true latent pseudotimes

By default this simulates the model for $N= 100$ cells and $G=40$ genes.

For 8 representative genes we can visualise what the expression looks like over pseudotime:

```{r some-genes, fig.width = 7}
genes_to_extract <- c(1,3,11,13,21,23,31,33)
expression_df <- as.data.frame(sim$y[,genes_to_extract])
names(expression_df) <- paste0("gene_", genes_to_extract)

df_gex <- as_tibble(expression_df) %>% 
  mutate(x = factor(sim[['x']]), z = sim[['z']]) %>% 
  gather(gene, expression, -x, -z)

ggplot(df_gex, aes(x = z, y = expression, color = x)) +
  geom_point() +
  facet_wrap(~ gene, nrow = 2) +
  scale_color_brewer(palette = "Set1")
```

We see for the first two genes there is differential expression only, genes 3 and 4 show a pseudotime dependence, genes 5 and 6 show pseudotime-covariate interactions (but marginally no differential expression), while genes 7 and 8 show differential expression, pseudotime dependence and pseudotime-covariate interactions.

We can further plot this in PCA space, coloured by both covariate and pseudotime to get an idea of how these look in the reduced dimension:

```{r pcaing, fig.show = 'hold'}
pca_df <- as_tibble(as.data.frame(prcomp(sim$y)$x[,1:2])) %>% 
  mutate(x = factor(sim[['x']]), z = sim[['z']])

ggplot(pca_df, aes(x = PC1, y = PC2, color = x)) +
  geom_point() + scale_colour_brewer(palette = "Set1")

ggplot(pca_df, aes(x = PC1, y = PC2, color = z)) +
  geom_point()
```


## Fit PhenoPath model

PhenoPath fits models with a call to the `phenopath` function, which requires at least an expression matrix `y` and a covariate vector `x`. The expression data should represent something comparable to logged counts, such as $log_2(\text{TPM}+1)$. Note that buy default PhenoPath centre-scales the input expression.

For use with `SummarizedExperiment`s see the [section on input formats](#inputdata). For this example we choose an ELBO tolerance of `1e-6` and ELBO calculations thinned by `40`. A discussion of how to control variational inference can be found [below](#varcontrol).

```{r see-results, cache=TRUE}
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-6, thin = 40)
print(fit)
```

The `phenopath` function will print progress on iterations, ELBO, and % change in ELBO that may be turned off by setting `verbose = FALSE` in the call.

Once the model has been fit it is important to check convergence with a call to `plot_elbo(fit)` to ensure the ELBO is approximately flat:

```{r plot-elbo}
plot_elbo(fit)
```

## Examining results


The posterior mean estimates of the pseudotimes $z$ sit in `fit$m_z` that can be extracted using the `trajectory` function. We can visualise these compared to both the true pseudotimes and the first principal component of the data:

```{r plot-results, fig.show = 'hold', fig.width = 2.5, fig.height = 2.5}
qplot(sim$z, trajectory(fit)) +
  xlab("True z") + ylab("Phenopath z")
qplot(sim$z, pca_df$PC1) +
  xlab("True z") + ylab("PC1")
```

We see that this has high correlation with the true pseudotimes:

```{r print-correlation}
cor(sim$z, trajectory(fit))
```

Next, we're interested in the interactions between the latent space and the covariates. There are three functions to help us here:

* `interaction_effects` retrieves the posterior interaction effect sizes
* `interaction_sds` retrieves the posterior interaction standard deviations
* `significant_interactions` applies a Bayesian significant test to the interaction parameters

Note that if $P=1$ (ie there is only one covariate) each of these will return a vector, while if $P>1$ then a matrix is returned. 

Alternatively, one can call the `interactions` function that returns a data frame with the following entries:

* `feature` The feature (usually gene)
* `covariate` The covariate, specified from the order originally supplied to the call to `phenopath`
* `interaction_effect_size` The effect size of the interaction ($\beta$ from the statistical model)
* `significant` Boolean for whether the interaction effect is significantly different from 0
* `chi` The precision of the ARD prior on $\beta$
* `pathway_loading` The pathway loading $\lambda$, showing the overall effect for each gene marginalised over the covariate

In our simulated data above, the first 20 genes were simulated with no interaction effect while the latter 20 were simulated with interaction effects. We can pull out the interaction effect sizes, standard deviations, and significance test results into a data frame and plot:

```{r beta-df, fig.width = 6, fig.height = 3}
gene_names <- paste0("gene", seq_len(ncol(fit$m_beta)))
df_beta <- data_frame(beta = interaction_effects(fit),
                      beta_sd = interaction_sds(fit),
                      is_sig = significant_interactions(fit),
                      gene = gene_names)

df_beta$gene <- fct_relevel(df_beta$gene, gene_names)

ggplot(df_beta, aes(x = gene, y = beta, color = is_sig)) + 
  geom_point() +
  geom_errorbar(aes(ymin = beta - 2 * beta_sd, ymax = beta + 2 * beta_sd)) +
  theme(axis.text.x = element_text(angle = 90, hjust = 1),
        axis.title.x = element_blank()) +
  ylab(expression(beta)) +
  scale_color_brewer(palette = "Set2", name = "Significant")
```

A typical analysis might follow by graphing the largest effect size; we can do this as follows:

```{r graph-largest-effect-size}
which_largest <- which.max(df_beta$beta)

df_large <- data_frame(
  y = sim[['y']][, which_largest],
  x = factor(sim[['x']]),
  z = sim[['z']]
)

ggplot(df_large, aes(x = z, y = y, color = x)) +
  geom_point() +
  scale_color_brewer(palette = "Set1") +
  stat_smooth()
```

# Advanced

## Using an `SummarizedExperiment` as input

Alternatively you might have expression values in an `SummarizedExperiment`. For single-cell data it is highly recommended to use the [SummarizedExperiment](https://bioconductor.org/packages/release/bioc/html/summarizedexperiment.html)  in which case data is stored in a class derived from `SummarizedExperiment`.

We'll first construct an example `SummarizedExperiment` using our previous simulation data:

```{r construct-sceset, warning = FALSE}
suppressPackageStartupMessages(library(SummarizedExperiment))
exprs_mat <- t(sim$y)
pdata <- data.frame(x = sim$x)
sce <- SummarizedExperiment(assays = list(exprs = exprs_mat), 
                            colData = pdata)
sce
```

Note that PhenoPath will use by default is in the `exprs` assay of a `SummarizedExperiment` (ie `assay(sce, "exprs")`) as gene expression values, which can be changed using the `sce_assay` string in the column to `phenopath`. 

We can then pass the $x$ covariates to PhenoPath in three ways:

1. As a vector or matrix as before
2. As a character that names a column of `colData(sce)` to use
3. A formula to build a model matrix from `colData(sce)`

For our example, these three look like

```{r example-using-expressionset, eval = FALSE}
fit <- phenopath(sce, sim$x) # 1
fit <- phenopath(sce, "x") # 2
fit <- phenopath(sce, ~ x) # 3
```

Note that if the column of the SCESet is a factor it is coerced into a one-hot encoding. The intercept term is then removed as this is taken care of by the $\lambda$ coefficient automatically, and the columns centre-scaled.

## Initialisation of latent space

The posterior distribution is naturally multi-modal and the use of variational inference means we essentially dive straight into the nearest local maximum. Therefore, correct initialisation of the latent space is important. This is controlled through the `z_init` argument.

PhenoPath allows for three different initialisations:

1. An integer specifying a principal component of the data to initialise to
2. A vector specifying the initial values
3. Random initialisation from standard normal distribution

For our example these three look like

```{r initialisation-examples, eval = FALSE}
fit <- phenopath(sim$y, sim$x, z_init = 1) # 1, initialise to first principal component
fit <- phenopath(sim$y, sim$x, z_init = sim$z) # 2, initialise to true values
fit <- phenopath(sim$y, sim$x, z_init = "random") # 3, random initialisation
```

## Controlling variational inference {#varcontrol}

There are several parameters that tune the coordinate ascent variational inference (CAVI):

1. `maxiter` maximum number of iterations to run CAVI for
2. `elbo_tol` the _percentage_ change in the ELBO below which the 
model is considered converged
3. `thin` Computing the ELBO is expensive, so only compute the ELBO every
`thin` iterations

For example:

```{r cavi-tuning, eval = FALSE}
fit <- phenopath(sim$y, sim$x,
                 maxiter = 1000, # 1000 iterations max
                 elbo_tol = 1e-2, # consider model converged when change in ELBO < 0.02%
                 thin = 20 # calculate ELBO every 20 iterations
                 )
```

# Technical

```{r sessioninfo}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clvm.R
\name{scale_vec}
\alias{scale_vec}
\title{Scale a vector to have mean 0 and variance 1}
\usage{
scale_vec(x)
}
\arguments{
\item{x}{Input vector to scale}
}
\value{
Scaled vector
}
\description{
Scales vector to mean 0 variance 1 unless input standard deviation is 0
in which case original vector is returned
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenopath.R
\name{interactions}
\alias{interactions}
\title{Tidy summary of interactions}
\usage{
interactions(phenopath_fit, n = 3)
}
\arguments{
\item{phenopath_fit}{An object returned by a call to \code{phenopath}}

\item{n}{The number of standard deviations away from 0 the posterior of the 
interaction effect sizes should be to be designated as significant}
}
\value{
A data frame with the following columns:
\itemize{
\item \code{feature} The feature (usually gene)
\item \code{covariate} The covariate, specified from the order originally supplied to 
the call to \code{phenopath}
\item \code{interaction_effect_size} The effect size of the interaction (\eqn{beta} 
from the statistical model)
\item \code{significant} Boolean for whether the interaction effect is significantly
different from 0
\item \code{chi} The precision of the ARD prior on \eqn{beta}
\item \code{pathway_loading} The pathway loading \eqn{lambda}, showing the overall
effect for each gene marginalised over the covariate
}
}
\description{
Computes a tidy data frame of the interaction effects, pathway scores, and significance for
each gene and covariate interaction.
}
\examples{
sim <- simulate_phenopath() 
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
interactions(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_elbo}
\alias{plot_elbo}
\title{Plots the ELBO}
\usage{
plot_elbo(fit)
}
\arguments{
\item{fit}{An object returned by a call to \code{phenopath}}
}
\value{
A \code{ggplot2} object of the ELBO against the number of iterations
}
\description{
Plots the evidence lower bound (ELBO) as a function of iterations
}
\examples{
sim <- simulate_phenopath()
fit <- phenopath(sim$y, sim$x)
plot_elbo(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenopath.R
\name{trajectory}
\alias{trajectory}
\title{Get the latent PhenoPath trajectory}
\usage{
trajectory(phenopath_fit)
}
\arguments{
\item{phenopath_fit}{An object of class \code{phenopath_fit}}
}
\value{
A vector of latent trajectory (pseudotime) values
}
\description{
Get the latent PhenoPath trajectory
}
\examples{
sim <- simulate_phenopath() 
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
z <- trajectory(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clvm.R
\name{clvm}
\alias{clvm}
\title{Fit a CLVM Model}
\usage{
clvm(y, x, maxiter = 10000, elbo_tol = 1e-05, thin = 1, verbose = TRUE,
  z_init = 1, tau_q = 1, tau_mu = 1, tau_c = 1, a = 2, b = 2,
  tau_alpha = 1, a_beta = 10, b_beta = 1, q = rep(0, nrow(y)),
  model_mu = FALSE, scale_y = TRUE)
}
\arguments{
\item{y}{A N-by-G (dynamic) input matrix}

\item{x}{A N-by-P (static) input matrix}

\item{maxiter}{Maximum number of CAVI iterations}

\item{elbo_tol}{The (percent) change in the ELBO below which it is 
considered converged}

\item{thin}{The number of iterations to wait each time before
re-calculating the elbo}

\item{verbose}{Print convergence messages}

\item{z_init}{The initialisation of the latent trajectory. Should be one of
\enumerate{
\item A positive integer describing which principal component of the data 
should be used for initialisation (default 1), \emph{or}
\item A numeric vector of length number of samples to be used directly 
for initialisation, \emph{or}
\item The text character \code{"random"}, for random initialisation from 
a standard normal distribution.
}}

\item{tau_q}{Hyperparameter tau_q}

\item{tau_mu}{Hyperparameter tau_mu}

\item{tau_c}{Hyperparameter tau_c}

\item{a}{Hyperparameter a}

\item{b}{Hyperparameter b}

\item{tau_alpha}{Hyperparameter tau_alpha}

\item{a_beta}{Hyperparameter a_beta}

\item{b_beta}{Hyperparameter b_beta}

\item{q}{Priors on the latent variables}

\item{model_mu}{Logical - should a gene-specific intercept term be modelled?}

\item{scale_y}{Logical - should the expression matrix be centre scaled?}
}
\value{
A list whose entries correspond to the converged values of the
variational parameters along with the ELBO.
}
\description{
Fit a covariate latent variable model using coordinate ascent
variational inference.
}
\examples{
sim <- simulate_phenopath()
fit <- clvm(sim$y, matrix(sim$x))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation.R
\name{simulate_phenopath}
\alias{simulate_phenopath}
\title{Simulate from phenopath model}
\usage{
simulate_phenopath(N = 100, G = 40, G_de = NULL, G_pst = NULL,
  G_pst_beta = NULL, G_de_pst_beta = NULL)
}
\arguments{
\item{N}{Number of samples to simulate}

\item{G}{Number of genes to simulate. Should be divisible by 4}

\item{G_de}{Number of genes to simulate from the \emph{differential expression} regime}

\item{G_pst}{Number of genes to simulate from the \emph{pseudotime} regime}

\item{G_pst_beta}{Number of genes to simulate from the \emph{pseudotime + beta interactions} regime}

\item{G_de_pst_beta}{Number of genes to simulate from the \emph{differential expression + pseudotime + interactions} regime}
}
\value{
A list with four entries:
\itemize{
\item \code{parameters} A \code{tibble} with an entry for each gene and a column
for each parameter value and simulation regime (see details).
\item \code{y} The N-by-G simulated gene expression matrix.
\item \code{x} The N-length vector of covariates.
\item \code{z} The N-length vector of pseudotimes.
}
}
\description{
Simulate synthetic data from the phenopath model, where each gene is sampled from 
one of four types  (see details).
}
\details{
Will simulate data for a number of genes corresponding to one of four regimes:
\enumerate{
\item \code{de} ('differential expression'), where the gene has no association to the latent 
trajectory and exhibits differential expression only
\item \code{pst} ('pseudotime'), the gene shows pseudotemporal regulation but no 
differential regulation
\item \code{pst_beta} ('pseudotime + beta interactions'), the gene shows pseudotemporal regulation
that is modulated by covariate interactions 
\item \code{de_pst_beta} ('differential expression + pseudotime + interactions), all of the above
}
}
\examples{
sim <- simulate_phenopath()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenopath.R
\name{phenopath}
\alias{phenopath}
\title{PhenoPath - genomic trajectories with heterogeneous backgrounds}
\usage{
phenopath(exprs_obj, x, sce_assay = "exprs", elbo_tol = 1e-05, z_init = 1,
  ...)
}
\arguments{
\item{exprs_obj}{Input gene expression, either
\enumerate{
\item An \linkS4class{SummarizedExperiment} object, \emph{or}
\item A cell-by-gene matrix of normalised expression values in log form.
}}

\item{x}{The covariate vector, either
\enumerate{
\item The name of a column of \code{colData(exprs_obj)} if \code{exprs_obj} is an
\code{SummarizedExperiment}, \emph{or}
\item A numeric of factor vector of length equal to the 
number of cells, \emph{or}
\item A formula from which to build a model matrix from \code{colData(exprs_obj)}, 
if \code{exprs_obj} is a \linkS4class{SummarizedExperiment}
}}

\item{sce_assay}{The assay from \code{exprs_obj} to use as expression if
\code{exprs_object} is a \code{SummarizedExperiment}}

\item{elbo_tol}{The relative pct change in the ELBO below 
which is considered converged.
See convergence section in details below.}

\item{z_init}{The initialisation of the latent trajectory. Should be one of
\enumerate{
\item A positive integer describing which principal component of the data should
be used for initialisation (default 1), \emph{or}
\item A numeric vector of length number of samples to be used 
directly for initialisation, \emph{or}
\item The text character \code{"random"}, for random initialisation 
from a standard normal distribution.
}}

\item{...}{Additional arguments to be passed to \code{\link{clvm}}. 
See description below for more details or call \code{?clvm}.}
}
\value{
An S3 structure with the following entries:
\itemize{
\item \code{m_z} The converged mean estimates of the trajectory
\item \code{s_z} The converged standard deviation estimates of z
\item \code{m_beta} A P-by-G matrix of interaction coefficients
\item \code{s_beta} A P-by-G matrix of interaction standard deviations
}
}
\description{
PhenoPath learns genomic trajectories in the presence of 
heterogenous environmental and genetic backgrounds. It takes 
input gene expression measurements that are modelled
by a single unobserved factor (the "trajectory"). The regulation 
of genes along the trajectory is perturbed by an additional set of 
covariates (such as genetic or environmental status) allowing for 
the identification of covariate-trajectory interactions.
The model is fitted using mean-field co-ordinate ascent variational inference.
}
\details{
\strong{Input expression}

If an \code{SummarizedExperiment} is provided, \code{assay(exprs_obj, sce_assay)} 
is used. 
This is assumed to be in
a form that is suitably normalised and approximately normal, such as 
the log of TPM values (plus a suitable offset) or 
similar.

\strong{Encoding covariates}

See the vignette.

\strong{Convergence of variational inference}

It is strongly recommended to call \code{plot_elbo(phenopath_fit)} after the fitting procedure to
ensure the ELBO has approximately converged (though convergence metrics are printed during the
fitting process). For a good introduction to variational inference see Blei, D.M., Kucukelbir, A. & McAuliffe, J.D., 2017. Variational Inference: A Review for Statisticians. Journal of the American Statistical Association.

\strong{Additional arguments}

Addition arguments to \code{clvm} are passed via \code{...}. For full documentation, call \code{?clvm}.
Some notable options:
\itemize{
\item \code{thin} - The ELBO is expensive to compute for larger datasets. The model will
compute the ELBO and compare convergence every \code{thin} iterations.
\item \code{q} and \code{tau_q} - Priors (such as capture times) for the latent space. Note that
\code{model_mu} should be true if \code{q} is non-zero.
\item \code{scale_y} By default the input expression is centre-scaled for each gene. If \code{scale_y}
is \code{FALSE} this does not happen - but note that \code{model_mu} should be \code{TRUE} in such a case.
}
}
\examples{
sim <- simulate_phenopath() # returns a list with gene expression in y and covariates in x
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)

# Extract the trajectory
z <- trajectory(fit)

}
\seealso{
\code{\link{clvm}} for the underlying CAVI function, \code{\link{trajectory}}
to extract the latent trajectory, \code{\link{interaction_effects}} for the interaction effect
sizes, \code{\link{significant_interactions}} for the results of Bayesian significance testing.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation.R
\name{sample_fns}
\alias{sample_fns}
\alias{sample_de}
\alias{sample_fns}
\alias{sample_pst}
\alias{sample_fns}
\alias{sample_pst_beta}
\alias{sample_fns}
\alias{sample_de_pst_beta}
\title{Sample parameters for simulation}
\usage{
sample_de()

sample_pst()

sample_pst_beta()

sample_de_pst_beta()
}
\value{
A length-3 vector of parameters corresponding to the particular 
simulation regime
}
\description{
Sample parameters from de regime

Sample parameters from pseudotime regime

Sample parameters from pseudotime-interaction regime

Sample paramters from de-pseudotime-interaction regime
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenopath.R
\name{significant_interactions}
\alias{significant_interactions}
\title{Significance testing for interaction features}
\usage{
significant_interactions(phenopath_fit, n = 3)
}
\arguments{
\item{phenopath_fit}{The results of a call to \code{clvm}}

\item{n}{The number of standard deviations away from 0 the posterior
estimate of beta should be to be designated significant.}
}
\value{
A logical vector describing whether each feature 
passes the significance test.
}
\description{
Given the results of \code{clvm}, decide which features show significant
iteractions between the latent trajectory and covariates. Significant 
features are designated as those where the variational mean of the interaction
coefficient falls outside the \eqn{n \sigma} interval of 0.
}
\examples{
sim <- simulate_phenopath() 
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
signints <- significant_interactions(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenopath.R
\name{interaction_effects}
\alias{interaction_effects}
\title{Get the interaction effect sizes}
\usage{
interaction_effects(phenopath_fit)
}
\arguments{
\item{phenopath_fit}{An object of class \code{phenopath_fit}}
}
\value{
TODO
}
\description{
Get the interaction effect sizes
}
\examples{
sim <- simulate_phenopath() 
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
beta <- interaction_effects(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation.R
\name{simulate_one_gene}
\alias{simulate_one_gene}
\title{Simulate one gene}
\usage{
simulate_one_gene(N, pst, x, alpha = 0, c = 0, beta = 0, tau = 1e+06)
}
\value{
A length-N gene expression vector simulated with the 
PhenoPath mean function for the given parameters
}
\description{
Simulate one gene from the model given paramters, z and covariates
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenopath.R
\name{interaction_sds}
\alias{interaction_sds}
\title{Get the interaction standard deviations}
\usage{
interaction_sds(phenopath_fit)
}
\arguments{
\item{phenopath_fit}{An object of class \code{phenopath_fit}}
}
\value{
TODO
}
\description{
Get the interaction standard deviations
}
\examples{
sim <- simulate_phenopath() 
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
beta_sd <- interaction_sds(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenopath.R
\name{print.phenopath_fit}
\alias{print.phenopath_fit}
\title{Print a PhenoPath fit}
\usage{
\method{print}{phenopath_fit}(x, ...)
}
\arguments{
\item{x}{A \code{phenopath_fit} returned by a call to \code{phenopath}}

\item{...}{Additional arguments}
}
\value{
A string representation of a \code{phenopath_fit} object.
}
\description{
Print a PhenoPath fit
}
\examples{
sim <- simulate_phenopath() 
fit <- phenopath(sim$y, sim$x, elbo_tol = 1e-2)
print(fit)
}
